#!/usr/bin/env python

doc="""
#  -----------------------------------------------------------
#
#   'ProcessAll.py'
#
#   v.7.3  by Evgenii Kovrigin      3/18/2005
#
#   Batch processing of multiple fids.
# 
#  -----------------------------------------------------------

    Usage: ProcessAll.py directory_table.txt [-ps] [-1 experiment_directory] [-color]

    - 'directory_list.txt' that supplies the directory names where to look for fids.
       Directory list file contains one line header and a column of names for directories
       you want to process.

       'ProcessAll.py' directly checks for the directories to exist
       and to contain the fid file to process and issues a warning if it is not found.

    - one can use 1 or 2 parallel processing schemes (set in nmrproc1.P1.com,  nmrproc1.P2.com)
      while phase shifting is done the same way on both of them (set in nmrproc2.com). Results
      are placed into Protocol_1/ and Protocol_2 directory. Number of protocols you want to use
      is set in ProcessAll.ini file.
      
    - The program works in the current directory
    
    - Current directory should contain:

        - 'ProcessAll.ini' file where all necessary parameters are set
        
        - check nmrproc1.P1.com,  nmrproc1.P2.com and nmrproc2.com macros
          for specifics of your processing needs

    - to process ADDED fids set <fid_name> 'final.fid' in ProcessAll.ini

    - For one fid processing use option: -1
       "ProcessAll.py rename_table.txt -1 experiment_directory"

    - For post-processing phase shifting use option: -ps

    - If you want to see colors use: -color

    - To do the phase shifting with individual set of phases for some datasets:
      create 'fid_dir_name.phases' text file in the 'data_path' directory
      and insert phases for direct and indirect dimensions as
      <Xp0> xp0
      <Xp1> xp1
      <Yp0> yp0
      <Yp1> yp1
      
      !!! Phases may be given as mathematical expressions, like 30+2.3 (NO SPACES!!!)
    
#  -----------------------------------------------------------

"""

import os
import sys
import string
import shutil

import mylib
import keyfile


#--- begin here ------

if len(sys.argv)<2:
	print doc
	sys.exit()

import colors
from colors import *
#-- to make it portable: use color only when requested explicitly
if '-color' in sys.argv: colors.COLOR=ON


#---- load parameters -----------
path_to_fids='./'
ini_filename="ProcessAll.ini"
inipars=keyfile.Keyword_File(path_to_fids,ini_filename)   


#---- load parameters -----------
experiment_type=inipars.get_string("<Experiment_type>")

temperature=inipars.get_string("<temperature_label>")

Xp0=eval(inipars.get_string("<Xp0>"))  # common phases: direct dimension
Xp1=eval(inipars.get_string("<Xp1>"))
Yp0=eval(inipars.get_string("<Yp0>"))  # common phases: indirect dimension
Yp1=eval(inipars.get_string("<Yp1>"))

protocols_to_run=inipars.get_integer("<protocols_num>")

fid_to_process=inipars.get_string("<fid_name>") 

#------- macros' definitions -----------
protocols=[ "nmrproc1.P1.com",
            "nmrproc1.P2.com"
          ]

if protocols_to_run>len(protocols):
    print "\a\a\aError! Maximum number of protocols is set to %s!!!" % len(protocols)
    sys.exit()

second_part_macro=os.path.join(path_to_fids,"nmrproc2.com")

#-----  make a directory list out of 'rename_table.txt' file --------
cprint(INV_RED,"-"*60)
print " BATCH PROCESSING OF %s " % experiment_type
cprint(INV_RED,"-"*60)

table=mylib.String_2_Words_2D(mylib.Load_Text_File(sys.argv[1]))
print '\n','-'*40,'\nDirectories to be processed:', 
dirlist=[]
for line in table[1:]:
        if len(line):
           new_name=string.strip(line[0])
           if '/' in new_name:
		   print "\n\nError!\n Please, remove '/' from the name '%s' in '%s'!!!\n\n" %(
			   new_name,sys.argv[1])
		   sys.exit()
		   
           #-- Check if this directory exists and contains necessary fid --
           if os.path.isdir(new_name):
                   if os.path.isfile(os.path.join(new_name,fid_to_process)):
                        #--- OK. Correct path ---
                        print new_name,
                        dirlist.append(new_name)
                        continue
	   #--- !!! Error in the directory name -----
           print "\n\nError!\nPath '%s/%s' does not exist! Check contents of '%s'!\n\n" %(
		   new_name,fid_to_process,sys.argv[1])
           sys.exit()
           
        
print '\n',40*'-','\n'

#--- check mode of work requested
Yes=1
No=0
DoJustPhase=No
if '-ps' in sys.argv : DoJustPhase=Yes


if '-1' in sys.argv:  # take next word after '-1'
    dirname=sys.argv[sys.argv.index('-1')+1]
    dirlist=[dirname]


#-------------- begin processing -----------------
for experiment in dirlist:

    cprint(INV, "\n*********** Experiment %s ***********" % experiment)

    #--- copy fid  ---------------    
    if DoJustPhase==No :
        #---copy separated and processed fid to working directory ------------
        shutil.copyfile(os.path.join(path_to_fids,experiment, fid_to_process), "for_nmrPipe")

        
    # --- nmrPipe processing ---------------
    for x in range(protocols_to_run):
	num=x+1
	print clr(UNDERLINE, "--------- Protocol %s ---------"%num)
        

        dirname='Protocol_%d/' % num
        nmrPipe_output_name=os.path.join(dirname,experiment+".for_nmrproc2")
        for_sparky_name=os.path.join(dirname,
		experiment_type+"."+temperature+"."+experiment+".P%s"% num)
        
        if DoJustPhase==No :
            #--- create a directory --------
            if not os.path.isdir(dirname): os.mkdir(dirname)
         
            #--- run 'nmrproc1.Px.com' ------
            print "Processing ",experiment

            protocol_to_run=os.path.join(path_to_fids,protocols[x])            
            os.system(protocol_to_run)
            
            os.rename("result_for_nmrproc2", nmrPipe_output_name)

        #-- shift phase ----
        print "Shifting phase of ", experiment
        #-- check if phases for this experiment a given separately
        phase_fname=experiment+'.phases'
        if os.path.isfile(os.path.join(path_to_fids,phase_fname)):
		phase_file=keyfile.Keyword_File(path_to_fids,phase_fname)
		Xphase0=eval(phase_file.get_string("<Xp0>"))
		Xphase1=eval(phase_file.get_string("<Xp1>"))
		Yphase0=eval(phase_file.get_string("<Yp0>"))  #  indirect dimension
		Yphase1=eval(phase_file.get_string("<Yp1>"))
		print "Use individual phases from '%s'\nXp0= %s\tXp1= %s\nYp0= %s\tYp1= %s" % (
			phase_fname, Xphase0, Xphase1, Yphase0, Yphase1)
	else:
		Xphase0=Xp0
		Xphase1=Xp1
		Yphase0=Yp0
		Yphase1=Yp1
		print "Use common phases from '%s'" % ini_filename

        os.system(second_part_macro +   " %s %s %s %s %s %s"
              % (nmrPipe_output_name, Xphase0, Xphase1, Yphase0, Yphase1, for_sparky_name))

#--- clean up a bit ------
if DoJustPhase==No :
    os.system('rm for_nmrPipe')        
    
mylib.beep()
cprint(BLINK, "\nDone...")
